import pandas as pd

from typing import List
from landmarks_manager import LandmarkManager
from fastapi.encoders import jsonable_encoder

from optimizer_v4 import solve_optimization
from refiner import refine_optimization
from structs.landmarks import Landmark
from structs.landmarktype import LandmarkType
from structs.preferences import Preferences, Preference


# Helper function to create a .txt file with results
def write_data(L: List[Landmark], file_name: str): 

    data = pd.DataFrame()
    i = 0

    for landmark in L :
        data[i] = jsonable_encoder(landmark)
        i += 1

    data.to_json(file_name, indent = 2, force_ascii=False)


def test4(coordinates: tuple[float, float]) -> List[Landmark]:

    manager = LandmarkManager()

    preferences = Preferences(
                    sightseeing=Preference(
                                  name='sightseeing', 
                                  type=LandmarkType(landmark_type='sightseeing'),
                                  score = 5),
                    nature=Preference(
                                  name='nature', 
                                  type=LandmarkType(landmark_type='nature'),
                                  score = 0),
                    shopping=Preference(
                                  name='shopping', 
                                  type=LandmarkType(landmark_type='shopping'),
                                  score = 0))


    # Create start and finish 
    start = Landmark(name='start', type=LandmarkType(landmark_type='start'), location=coordinates, osm_type='start', osm_id=0, attractiveness=0, must_do=False, n_tags = 0)
    finish = Landmark(name='finish', type=LandmarkType(landmark_type='finish'), location=coordinates, osm_type='finish', osm_id=0, attractiveness=0, must_do=False, n_tags = 0)
    #finish = Landmark(name='finish', type=LandmarkType(landmark_type='finish'), location=(48.8777055, 2.3640967), osm_type='finish', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    #start = Landmark(name='start', type=LandmarkType(landmark_type='start'), location=(48.847132, 2.312359), osm_type='start', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    #finish = Landmark(name='finish', type=LandmarkType(landmark_type='finish'), location=(48.843185, 2.344533), osm_type='finish', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    #finish = Landmark(name='finish', type=LandmarkType(landmark_type='finish'), location=(48.847132, 2.312359), osm_type='finish', osm_id=0, attractiveness=0, must_do=True, n_tags = 0)
    
    # Generate the landmarks from the start location
    landmarks, landmarks_short = generate_landmarks(preferences=preferences, coordinates=start.location)
    #write_data(landmarks, "landmarks_Wien.txt")

    # Insert start and finish to the landmarks list
    landmarks_short.insert(0, start)
    landmarks_short.append(finish)

    max_walking_time = 180      # minutes
    detour = 0                 # minutes

    # First stage optimization
    base_tour = solve_optimization(landmarks_short, max_walking_time, True)

    # Second stage using linear optimization
    
    refined_tour = refine_optimization(landmarks, base_tour, max_walking_time, detour, True)


    return refined_tour


#test4(tuple((48.8344400, 2.3220540)))       # Café Chez César 
#test4(tuple((48.8375946, 2.2949904)))       # Point random
#test4(tuple((47.377859, 8.540585)))         # Zurich HB
#test4(tuple((45.7576485, 4.8330241)))      # Lyon Bellecour
test4(tuple((48.5848435, 7.7332974)))      # Strasbourg Gare
#test4(tuple((48.2067858, 16.3692340)))      # Vienne
